/*
  Web Access Counter for Linux/UNIX : Ccnt.c
  Copyright (C) 2000-2011 : Koichi Takatoi.

  This program is free licence.
  Exemption matters. 
  Even if you use this program and the trouble is absorbed, I do not take the responsibility. 

  - ABOUT SETTING -
  1) Make the directory of counter preservation.
     Set write permission of this directory.
     > mkdir /var/www/data
     > chmod 0777 /var/www/data

  *) Change the program if you define directory other than default. 
     The definition of PATH_COUNT is changed. 
     The definition of PATH_LOG is changed. 

  2) Compile this program and put "cgi-bin" directory.
     > gcc -o /var/www/cgi-bin/Ccnt.cgi Ccnt.c

  - USAGE -
  Write the following tag in the html file. 
  <img src="http://YOURSERVER/cgi-bin/Ccnt.cgi?[Parameter]">

  Set parameter as follows. 
  <Required parameter>
  SG=CounterName : Counter identification name.

  <Optional parameter>
  F1=RRGGBB(HexRGB) : Color of dent on the upper left of frame.
  F2=RRGGBB(HexRGB) : Color of frame.
  F3=RRGGBB(HexRGB) : Color of dent on the down right of frame.
  BG=RRGGBB(HexRGB) : Background color.
  N1=RRGGBB(HexRGB) : Color of counter.
  N2=RRGGBB(HexRGB) : Color of moving counter.
  TS=Number : Counter value for display test.

  The terminal character of an individual parameter is a semicolon.
  Sample) .../Ccnt.cgi?SG=FirstCounter;F2=FF6666;N1=FF0000;N2=CCCCCC;
  In this sample, the saved file on the counter value becomes FirstCounter.
  Frame color is orange, moving counter is gray and counter is red.

  - SPECIFICATION -
  The counter file is made by the first access.
  The creating permission of the file by HTTPD is necessary in the first access.
  You may deprive of the write permission of the directory when the counter file already exists.

  The value of the counter is accuracy in signed integer. 
  The movement of the counter that exceeds accuracy is not secured. 
  When the counter image is an error, it is as follows. 
  ERROR  1 : QUERYSTING acquisition failure. 
  ERROR  2 : Open failure of counter file. 
  ERROR  3 : Seek failure of counter file.
  ERROR  4 : Read failure of counter file.
  ERROR  5 : Write failure of counter file.
  ERROR  6 : The parameter definition name is not correct. 
  ERROR  7 : The value set to the parameter is not correct. 
  ERROR  8 : No permission that change directory to the counter preservation directory.
  ERROR  9 : Making failure of counter file. 
  ERROR 10 : The definition error is SG of the required parameter. 
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <time.h>
#include <signal.h>

#define FD_STDOUT	1
#define	PATH_COUNT	"/var/www/data"
#define	PATH_LOG	"/var/www/data/access.log"
#define	TIMEOUT		30

#define	SZ_COLOR_F	24
static unsigned char	FrameColor[SZ_COLOR_F] =
{	0xFF, 0xFF, 0xFF, 0xB0, 0xC0, 0xF0, 0x60, 0x80,
	0xE0, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
#define	SZ_COLOR_N	12
static unsigned char	Num1Color[SZ_COLOR_N] =
{	0xFF, 0xFF, 0xFF, 0x80, 0xFF, 0xFF, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00 };
static unsigned char	Num2Color[SZ_COLOR_N] =
{	0xFF, 0xFF, 0xFF, 0xE0, 0xE0, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00 };

#define	SZ_FRAMELEFT	50
static unsigned char	FrameLeft[SZ_FRAMELEFT] =
{	0x03, 0x2F, 0x18, 0xBA, 0x2B, 0x32, 0x83, 0x28,
	0x07, 0x65, 0xA0, 0x71, 0x3E, 0xA8, 0x6B, 0xCF,
	0x97, 0xE3, 0x68, 0xE2, 0x67, 0x61, 0xE7, 0x96,
	0x7A, 0x21, 0x4B, 0x2D, 0x50, 0x3C, 0x30, 0xB4,
	0x38, 0xB6, 0x1C, 0xBE, 0x82, 0xA8, 0xE9, 0xEB,
	0xBF, 0x1E, 0x70, 0x78, 0x6B, 0xD8, 0x64, 0xB1,
	0x04, 0x00 };
#define	SZ_FRAMERIGHT	53
static unsigned char	FrameRight[SZ_FRAMERIGHT] =
{	0x03, 0x32, 0x18, 0xBA, 0x41, 0x2A, 0x62, 0x0C,
	0x17, 0xA0, 0xA4, 0x56, 0xCA, 0x8C, 0xE3, 0x7C,
	0x1E, 0x24, 0x6E, 0xA2, 0xD7, 0x5D, 0x20, 0x5A,
	0x99, 0xE9, 0xB7, 0x4A, 0x4B, 0xA6, 0xCD, 0x8C,
	0x32, 0x94, 0x1E, 0xCC, 0xB5, 0xA7, 0xBB, 0xBF,
	0x3E, 0x16, 0xB0, 0x47, 0xE4, 0xB5, 0x62, 0xA1,
	0xD2, 0x6C, 0x99, 0x00, 0x00 };
#define	SZ_FRAMEMID		47
static unsigned char	FrameMid[SZ_FRAMEMID] =
{	0x03, 0x2C, 0x18, 0xBA, 0xBC, 0xF4, 0x30, 0xCA,
	0x49, 0x2B, 0x1C, 0x38, 0xEB, 0x6C, 0x3B, 0x15,
	0x60, 0x28, 0x8E, 0xE4, 0xE8, 0x9D, 0x68, 0xAA,
	0x36, 0xAC, 0xB2, 0xBD, 0x58, 0xCB, 0x96, 0x24,
	0xFC, 0xAA, 0x78, 0xAE, 0xEF, 0x7C, 0x1F, 0xC9,
	0x0D, 0xDA, 0xC8, 0xB6, 0x49, 0x00, 0x00 };

#define	SZ_NUM0			36
static unsigned char	Num0[SZ_NUM0] =
{	0x02, 0x21, 0x8C, 0x7F, 0x12, 0xC2, 0xED, 0x8C,
	0x90, 0x4A, 0x2E, 0xAE, 0x9A, 0xE2, 0x94, 0x98,
	0x37, 0x2D, 0x65, 0xD6, 0x76, 0x75, 0xD4, 0x67,
	0x28, 0x0F, 0xC4, 0xAE, 0xCB, 0x09, 0x69, 0x26,
	0x99, 0x1A, 0x05, 0x00 };
#define	SZ_NUM1			29
static unsigned char	Num1[SZ_NUM1] =
{	0x02, 0x1A, 0x94, 0x8F, 0x81, 0xCB, 0xAC, 0x1D,
	0x62, 0x13, 0xF1, 0x39, 0x09, 0xF1, 0xB5, 0xAB,
	0x4E, 0xCA, 0x25, 0x5A, 0x37, 0x76, 0x1F, 0xF5,
	0x79, 0x59, 0x28, 0x16, 0x00 };
#define	SZ_NUM2			35
static unsigned char	Num2[SZ_NUM2] =
{	0x02, 0x20, 0x8C, 0x7F, 0x12, 0xA2, 0x68, 0xDC,
	0xDC, 0x82, 0x94, 0xC9, 0x5A, 0x5D, 0xC4, 0x56,
	0xF3, 0x87, 0x64, 0xD7, 0xF3, 0x91, 0x58, 0xD2,
	0x25, 0xD3, 0xE2, 0x6D, 0xE5, 0x74, 0x46, 0xA8,
	0x50, 0x00, 0x00 };
#define	SZ_NUM3			37
static unsigned char	Num3[SZ_NUM3] =
{	0x02, 0x22, 0x8C, 0x7F, 0x12, 0xA2, 0x68, 0xDC,
	0xDC, 0x5A, 0xB0, 0xA6, 0x0A, 0xAF, 0x8B, 0x38,
	0xBF, 0xCE, 0x31, 0x08, 0x27, 0x4D, 0x54, 0x77,
	0xA1, 0x5F, 0xF4, 0x9C, 0xEC, 0x04, 0x8A, 0x2B,
	0xA6, 0x6D, 0x46, 0x01, 0x00 };
#define	SZ_NUM4			34
static unsigned char	Num4[SZ_NUM4] =
{	0x02, 0x1F, 0x94, 0x8F, 0x61, 0xC1, 0xAD, 0x08,
	0x57, 0x83, 0x33, 0x0A, 0x67, 0x1D, 0xA3, 0x3B,
	0xF7, 0x58, 0x79, 0xDF, 0xA1, 0x3D, 0xE4, 0xA6,
	0x68, 0x57, 0x82, 0x99, 0xE0, 0xC8, 0xBA, 0x5C,
	0x01, 0x00 };
#define	SZ_NUM5			34
static unsigned char	Num5[SZ_NUM5] =
{	0x02, 0x1F, 0x8C, 0x7F, 0xA2, 0x82, 0xA9, 0x9D,
	0xD0, 0x9A, 0x74, 0xCA, 0x66, 0x4D, 0xDD, 0xCB,
	0xB1, 0xA8, 0x41, 0x41, 0x37, 0x72, 0x5E, 0x35,
	0x62, 0x14, 0x56, 0x6E, 0x67, 0x56, 0x42, 0x5F,
	0x01, 0x00 };
#define	SZ_NUM6			36
static unsigned char	Num6[SZ_NUM6] =
{	0x02, 0x21, 0x8C, 0x7F, 0xA2, 0x82, 0xA9, 0x9D,
	0xD0, 0x9A, 0xCC, 0x50, 0xD9, 0xA6, 0xA3, 0x35,
	0x5C, 0x1C, 0x59, 0x90, 0xF7, 0x90, 0x5C, 0x77,
	0x3E, 0x9D, 0xB5, 0x78, 0x99, 0xCB, 0xB6, 0x99,
	0x16, 0x82, 0x05, 0x00 };
#define	SZ_NUM7			32
static unsigned char	Num7[SZ_NUM7] =
{	0x02, 0x1D, 0x8C, 0x7F, 0x12, 0xC2, 0xED, 0x0F,
	0x17, 0x54, 0x66, 0x52, 0x89, 0x56, 0x9A, 0x3B,
	0x56, 0x5E, 0x65, 0xA1, 0xE4, 0x91, 0x4F, 0x17,
	0x59, 0x8C, 0x79, 0x7E, 0xA5, 0x6A, 0x14, 0x00 };
#define	SZ_NUM8			35
static unsigned char	Num8[SZ_NUM8] =
{	0x02, 0x20, 0x8C, 0x7F, 0x12, 0xA2, 0x68, 0xDC,
	0xDC, 0x42, 0x90, 0xC9, 0x1A, 0x27, 0xBD, 0x95,
	0x43, 0x97, 0x45, 0x59, 0x42, 0x61, 0x9E, 0xF5,
	0x68, 0xA4, 0x35, 0x2A, 0x62, 0x07, 0xC2, 0x29,
	0x68, 0x14, 0x00 };
#define	SZ_NUM9			37
static unsigned char	Num9[SZ_NUM9] =
{	0x02, 0x22, 0x8C, 0x7F, 0x12, 0xA2, 0x68, 0xDC,
	0xDC, 0x42, 0x90, 0xA5, 0x0A, 0xA9, 0x8B, 0x37,
	0x3F, 0x6C, 0x7D, 0x5C, 0xC4, 0x25, 0x14, 0x26,
	0xA1, 0xCF, 0xA4, 0x55, 0xDB, 0x02, 0x86, 0x30,
	0x78, 0xA5, 0x46, 0x01, 0x00 };

#define	SZ_ERR			165
static unsigned char	Err[SZ_ERR] =
{	0x02, 0xA2, 0x54, 0x8E, 0x68, 0x62, 0xA9, 0xCC,
	0xDC, 0x83, 0x52, 0x85, 0x58, 0xA1, 0xCE, 0xBA,
	0xB9, 0xEE, 0x25, 0xE0, 0x08, 0x72, 0x54, 0x76,
	0x61, 0xDD, 0x95, 0x36, 0xEB, 0x82, 0x49, 0xB0,
	0xBB, 0xC1, 0xD3, 0x78, 0x7B, 0xF1, 0x7A, 0xEC,
	0x5B, 0xAF, 0xFB, 0xA5, 0x72, 0xBC, 0x96, 0x2F,
	0xF7, 0x40, 0x02, 0x95, 0xC3, 0x10, 0x6B, 0xE9,
	0xA4, 0x2D, 0x3E, 0xC0, 0xA9, 0x28, 0x19, 0xAD,
	0x12, 0x85, 0x45, 0xDF, 0x69, 0x58, 0x3D, 0x86,
	0x83, 0xDB, 0x6F, 0x37, 0x68, 0x46, 0x8B, 0x8D,
	0xE4, 0x67, 0x2F, 0xC9, 0x6E, 0x97, 0x96, 0x6E,
	0xB7, 0xEA, 0xEC, 0x7C, 0xB7, 0x50, 0xC2, 0x6B,
	0xDF, 0xB2, 0x27, 0x13, 0x48, 0x72, 0x32, 0x07,
	0x66, 0x78, 0xC7, 0x25, 0x95, 0x56, 0xE6, 0xB5,
	0x76, 0xC6, 0xE4, 0x88, 0x56, 0x17, 0x47, 0x39,
	0x63, 0x65, 0x51, 0x53, 0x52, 0x43, 0xC5, 0x29,
	0xF2, 0xD2, 0x49, 0x91, 0xA5, 0xF9, 0x22, 0x1A,
	0xAA, 0x86, 0xA8, 0x18, 0x89, 0x0A, 0xC7, 0xD6,
	0x4A, 0xC8, 0x4A, 0xE7, 0x2A, 0x0B, 0x39, 0x86,
	0x25, 0xD8, 0x54, 0x99, 0x3B, 0x17, 0x91, 0x47,
	0xC3, 0xC2, 0x5B, 0x00, 0x00 };

#define	SZ_IMGEND		34
static unsigned char	ImgEnd[SZ_IMGEND] =
{ 0x21, 0xFE, 0x1D, 0x43, 0x6F, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74,
  0x20, 0x28, 0x43, 0x29, 0x20, 0x4B, 0x6F, 0x69, 0x63, 0x68, 0x69, 0x20,
  0x54, 0x61, 0x6B, 0x61, 0x74, 0x6F, 0x69, 0x2E, 0x00, 0x3B };

#define	K_COLOR_F		0
#define	K_COLOR_1		1
#define	K_COLOR_2		2
#define	K_FRAMELEFT		0
#define	K_FRAMERIGHT	1
#define	K_FRAMEMID		2
#define	KETA_DEFAULT	4
#define	KETA_MAX		10

static int				SzColor[3] =
{	SZ_COLOR_F, SZ_COLOR_N, SZ_COLOR_N };
static unsigned char	*DtColor[3] =
{	FrameColor, Num1Color, Num2Color };

static int				SzFrame[3] =
{	SZ_FRAMELEFT, SZ_FRAMERIGHT, SZ_FRAMEMID };
static unsigned char	*DtFrame[3] =
{	FrameLeft, FrameRight, FrameMid };
static unsigned short	WiFrame[3] =
{	0x0007, 0x0007, 0x000D };

static int				SzNum[10] =
{	SZ_NUM0, SZ_NUM1, SZ_NUM2, SZ_NUM3, SZ_NUM4,
	SZ_NUM5, SZ_NUM6, SZ_NUM7, SZ_NUM8, SZ_NUM9 };
static unsigned char	*DtNum[10] =
{	Num0, Num1, Num2, Num3, Num4, Num5, Num6, Num7, Num8, Num9 };

extern	int		GetParameter(unsigned long *);
extern	int		GetCounter(unsigned long *);
extern	void	WriteStart(int);
extern	void	WriteHead(unsigned short);
extern	void	WriteColor(int);
extern	void	WriteFrame(int, unsigned short);
extern	void	WriteNumber(int, int, unsigned short);
extern	void	WriteControl(unsigned short);
extern	void	WriteEnd();
extern	void	NumberDsp(int, int *);
extern	void	ErrDsp(int);
extern	int		FLockOpen(char *, int, void (*));
extern	void	FLockClose(int);
extern	void	TmFunc();
extern	void	GetCookie();
extern	void	PutCookie(time_t);
extern	char	*GetNextData(char *, char);

char	*SG = (char *)0;
char	*AC = (char *)0;
int		CtMove = 1;

#define	SUCCESS		0
#define	ERR_QUERY	1
#define	ERR_OPEN	2
#define	ERR_SEEK	3
#define	ERR_READ	4
#define	ERR_WRITE	5
#define	ERR_LOPT	6
#define	ERR_ROPT	7
#define	ERR_CHDIR	8
#define	ERR_CREAT	9
#define	ERR_TAG		10

static char *Content_type = "Content-type: image/gif\n\n";
#define		CONTENT_SIZE	25

/*----	main			----*/
main()
{
	unsigned long	count = 0;
	unsigned long	base;
	int				number[KETA_MAX];
	int				ct;
	int				keta;
	int				ret;
	int				sz;
	time_t			tt;

	if((ret = GetParameter(&count)) != SUCCESS) {
		write(FD_STDOUT, Content_type, CONTENT_SIZE);
		ErrDsp(ret);
	}

	if(count == 0) {
		if(SG == (char *)0) {
			write(FD_STDOUT, Content_type, CONTENT_SIZE);
			ErrDsp(ERR_TAG);
		}
		GetCookie();
		if(AC != (char *)0) {
			sz = strlen(AC);
			if(sz == strlen(SG)) {
				if(memcmp(AC, SG, sz) == 0) {
					tt = time(0);
					PutCookie(tt);
					CtMove = 0;
				}
			}
		}
		if((ret = GetCounter(&count)) != SUCCESS) {
			write(FD_STDOUT, Content_type, CONTENT_SIZE);
			ErrDsp(ret);
		}
	}

	write(FD_STDOUT, Content_type, CONTENT_SIZE);
	for(ct = 0; ct < KETA_MAX; ct++) {
		if(ct == 0) {
			base = 1;
		} else {
			base *= 10;
		}
		if(count >= base) {
			keta = ct;
		}
		number[ct] = (count / base) % 10;
	}
	if(keta < KETA_DEFAULT) {
		keta = KETA_DEFAULT;
	}

	NumberDsp(keta, number);
}
/*----	GetParameter	----*/
int		GetParameter(unsigned long *count)
{
	char			*param;
	int				len;
	int				ct = 0;
	int				sts = 0;
	int				ct2;
	unsigned char	col[3];

	(*count) = 0;

	if((param = getenv("QUERY_STRING")) == (char *)0) {
		return(ERR_QUERY);
	}
	len = strlen(param);
	while(ct < len) {
		if((ct + 3) >= len) {
			return(ERR_LOPT);							/* E-5	short tag	*/
		}
		if(param[ct + 2] != '=') {
			return(ERR_LOPT);							/* E-5	= nothing	*/
		}
		if(memcmp(&param[ct], "TS", 2) == 0 || memcmp(&param[ct], "ts", 2) == 0) {
			(*count) = 0;
			ct += 3;
			if(param[ct] == 'E' || param[ct] == 'e') {
				sts = 0;
				ct++;
			} else {
				sts = 1;
			}
			while((ct < len) && (param[ct] != ';')) {
				(*count) *= 10;
				if(param[ct] < '0' || param[ct] > '9') {
					return(ERR_ROPT);					/* E-6	Invalid num	*/
				}
				(*count) += (unsigned long)(param[ct] - '0');
				ct++;
			}
			if(sts == 0) {
				return((*count) % 100);					/* Error number dsp	*/
			}
		} else if(memcmp(&param[ct], "SG", 2) == 0 || memcmp(&param[ct], "sg", 2) == 0) {
			ct += 3;
			SG = &param[ct];
			while((ct < len) && (param[ct] != ';')) {
				ct++;
			}
			if(param[ct] == ';') {
				param[ct] = '\0';
			}
		} else if(memcmp(&param[ct], "AC", 2) == 0 || memcmp(&param[ct], "ac", 2) == 0) {
			ct += 3;
			AC = &param[ct];
			while((ct < len) && (param[ct] != ';')) {
				ct++;
			}
			if(param[ct] == ';') {
				param[ct] = '\0';
			}
		} else {
			if(memcmp(&param[ct], "BG", 2) == 0 || memcmp(&param[ct], "bg", 2) == 0) {
				sts = 0;
				ct += 3;
			} else if(param[ct] == 'F' || param[ct] == 'f') {
				ct++;
				if(param[ct]  < '1' || param[ct] > '3') {
					return(ERR_LOPT);					/* E-5	Out of num	*/
				}
				sts = (int)(param[ct] - '0');
				ct += 2;
			} else if(param[ct] == 'N' || param[ct] == 'n') {
				ct++;
				if(param[ct] != '1' && param[ct] != '2') {
					return(ERR_LOPT);					/* E-5	Out of num	*/
				}
				sts = ((int)(param[ct] - '0')) + 3;
				ct += 2;
			} else {
				return(ERR_LOPT);						/* E-5	Unknown tag	*/
			}
			if((ct + 7) > len) {
				return(ERR_ROPT);						/* E-6	Short data	*/
			}
			for(ct2 = 0; ct2 < 6; ct2++) {
				if(param[ct] >= '0' && param[ct] <= '9') {
					param[ct] -= '0';
				} else if(param[ct] >= 'A' && param[ct] <= 'F') {
					param[ct] -= ('A' - (char)10);
				} else if(param[ct] >= 'a' && param[ct] <= 'f') {
					param[ct] -= ('a' - (char)10);
				} else {
					return(ERR_ROPT);					/* E-6	Out of hex	*/
				}
				if((ct2 % 2) == 0) {
					col[ct2 / 2] = (unsigned char)param[ct] * (unsigned char)0x10;
				} else {
					col[ct2 / 2] += (unsigned char)param[ct];
				}
				ct++;
			}
			if(param[ct] != ';') {
				return(ERR_ROPT);						/* E-6 ; nothing	*/
			}
			switch(sts) {
			  case 0:	/* BG */
				memcpy(&FrameColor[12], col, 3);
				memcpy(&Num1Color[6], col, 3);
				memcpy(&Num2Color[6], col, 3);
				break;
			  case 1:	/* F1 */
			  case 2:	/* F2 */
			  case 3:	/* F3 */
				memcpy(&FrameColor[(3 * sts)], col, 3);
				break;
			  case 4:	/* N1 */
				memcpy(&Num1Color[3], col, 3);
				break;
			  case 5:	/* N2 */
				memcpy(&Num2Color[3], col, 3); 
				break;
			  default:
				break;
			}
		}
		ct++;
	}
	return(SUCCESS);
}
/*----	GetCounter		----*/
int		GetCounter(unsigned long *count)
{
	char			buf[12];
	char			*ref;
	char			*rem;
	int				fd;
	FILE			*fp;
	time_t			tm;
	struct tm		*stm;
	char			*ctm;

	memset(buf, 0, 12);
	ref = getenv("HTTP_REFERER");

	if(chdir(PATH_COUNT) == -1) {
		return(ERR_CHDIR);
	}

	if(access(SG, F_OK) != 0) {
		if((fd = creat(SG, 0666)) == -1) {
			return(ERR_CREAT);
		}
		close(fd);
		if(ref != (char *)0) {
			if((fp = fopen(".new", "a")) != (FILE *)0) {
				fprintf(fp, "%s(%s)\n", ref, SG);
				fclose(fp);
			}
		}
	}

	if((fd = FLockOpen(SG, TIMEOUT, TmFunc)) == -1) {
		return(ERR_OPEN);
	}
	if(read(fd, buf, 10) == -1) {
		FLockClose(fd);
		return(ERR_READ);
	}
	if(lseek(fd, 0, SEEK_SET) == -1) {
		FLockClose(fd);
		return(ERR_SEEK);
	}
	(*count) = atol(buf);
	if(*count != 0xffffffff) {
		(*count) += CtMove;
	}
	sprintf(buf, "%d", *count);
	if(write(fd, buf, strlen(buf)) == -1) {
		FLockClose(fd);
		return(ERR_WRITE);
	}
	FLockClose(fd);

	if((rem = getenv("REMOTE_HOST")) == (char *)NULL) {
		if((rem = getenv("REMOTE_ADDR")) == (char *)NULL) {
			rem = "Unknown";
		}
	}
	tm = time(0);
	stm = localtime(&tm);
	ctm = asctime(stm);

	if((fp = fopen(PATH_LOG, "a")) != (FILE *)0) {
		fprintf(fp, "%s %d %.*s %s\n", SG, *count, strlen(ctm) -1, ctm, rem);
		fclose(fp);
	}

	return(SUCCESS);
}
/*----	WriteStart		----*/
void	WriteStart(int keta)
{
	unsigned short	width;
	int				ct;

	WriteHead((WiFrame[K_FRAMELEFT] + WiFrame[K_FRAMERIGHT]
	  + (WiFrame[K_FRAMEMID] * (keta + 1))));
	WriteColor(K_COLOR_F);

	WriteFrame(K_FRAMELEFT, 0);
	width = WiFrame[K_FRAMELEFT];
	for(ct = 0; ct <= keta; ct++) {
		WriteFrame(K_FRAMEMID, width);
		width += WiFrame[K_FRAMEMID];
	}
	WriteFrame(K_FRAMERIGHT, width);
}
/*----	WriteHead		----*/
void	WriteHead(unsigned short width)
{
	static unsigned char	filehead[13] =
	{	0x47, 0x49, 0x46, 0x38, 0x39, 0x61,	0xFF, 0xFF,
		0x20, 0x00, 0xA2, 0xFF, 0x00 };

	filehead[6] = (unsigned char)(width % 0x100);
	filehead[7] = (unsigned char)(width / 0x100);

	write(FD_STDOUT, filehead, 13);
}
/*----	WriteColor		----*/
void	WriteColor(int kind)
{
	write(FD_STDOUT, DtColor[kind], SzColor[kind]);
}
/*----	WriteFrame		----*/
void	WriteFrame(int kindfrm, unsigned short left)
{
	static unsigned char	imghead[10] =
	{  0x2C, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x20, 0x00, 0x40 };

	imghead[1] = (unsigned char)(left % 0x100);
	imghead[2] = (unsigned char)(left / 0x100);
	imghead[5] = (unsigned char)(WiFrame[kindfrm] % 0x100);
	imghead[6] = (unsigned char)(WiFrame[kindfrm] / 0x100);
	write(FD_STDOUT, imghead, 10);
	write(FD_STDOUT, DtFrame[kindfrm], SzFrame[kindfrm]);
}
/*----	WriteNumber		----*/
void	WriteNumber(int number, int kindcol, unsigned short left)
{
	static unsigned char	imghead[10] =
	{  0x2C, 0xFF, 0xFF, 0x08, 0x00, 0x0B, 0x00, 0x11, 0x00, 0xC1 };
	imghead[1] = (unsigned char)(left % 0x100);
	imghead[2] = (unsigned char)(left / 0x100);
	write(FD_STDOUT, imghead, 10);
	WriteColor(kindcol);
	write(FD_STDOUT, DtNum[number], SzNum[number]);
}
/*----	WriteControl	----*/
void	WriteControl(unsigned short delay)
{
	static unsigned char	control[8] = 
	{ 0x21, 0xF9, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00 };
	control[4] = (unsigned char)(delay % 0x100);
	control[5] = (unsigned char)(delay / 0x100);
	write(FD_STDOUT, control, 8);
}
/*----	WriteEnd		----*/
void	WriteEnd()
{
	write(FD_STDOUT, ImgEnd, SZ_IMGEND);
}
/*-----	NumberDsp		----*/
void	NumberDsp(int keta, int *number)
{
	int				ct;
	unsigned short	width;

	WriteStart(keta);

	width = WiFrame[K_FRAMELEFT];
	for(ct = keta; ct > 0; ct--) {
		WriteControl(50);
		WriteNumber(number[ct], K_COLOR_2, width + 1);
		WriteControl(0);
		WriteNumber(number[ct], K_COLOR_1, width + 1);
		width += WiFrame[K_FRAMEMID];
	}
	width += 1;
	for(ct = 0; ct < 10; ct++) {
		WriteNumber(ct, K_COLOR_2, width);
	}
	for(ct = 0; ct < number[0]; ct++) {
		WriteNumber(ct, K_COLOR_2, width);
	}
	WriteNumber(number[0], K_COLOR_1, width);

	WriteEnd();
}
/*----	ErrDsp			----*/
void	ErrDsp(int errnumber)
{
	unsigned short			width;
	static unsigned char	imghead[10] =
	{  0x2C, 0xFF, 0xFF, 0x08, 0x00, 0x41, 0x00, 0x11, 0x00, 0xC1 };

	WriteStart(7);

	width = WiFrame[K_FRAMELEFT];
	imghead[1] = (unsigned char)((width + 1) % 0x100);
	imghead[2] = (unsigned char)((width + 1) / 0x100);
	write(FD_STDOUT, imghead, 10);
	WriteColor(K_COLOR_1);
	write(FD_STDOUT, Err, SZ_ERR);

	width += (WiFrame[K_FRAMEMID] * 6);
	WriteNumber(((errnumber / 10) % 10), K_COLOR_1, width + 1);
	width += WiFrame[K_FRAMEMID];
	WriteNumber((errnumber % 10), K_COLOR_1, width + 1);

	WriteEnd();
	exit(-1);
}

/*----	FLockOpen		----*/
int		FLockOpen(char *fname, int timeout, void *func)
{
	struct flock	lock;
	int				fd;

	lock.l_type = F_WRLCK;
	lock.l_start = 0;
	lock.l_whence = SEEK_SET;
	lock.l_len = 0;

	if((fd = open(fname, O_RDWR, 0606)) == -1) {
		return(-1);
	}
	signal(SIGALRM, func);
	signal(SIGTERM, func);	/* apache mother process stop. child ??		*/
	signal(SIGHUP,	func);	/* apache mother process restart. child ??	*/
	signal(SIGINT,	func);
	signal(SIGKILL,	func);

	alarm(timeout);
	if(fcntl(fd, F_SETLKW, &lock) == -1) {
		close(fd);
		return(-1);
	}
	alarm(0);
	return(fd);
}
/*----	FLockClose		----*/
void	FLockClose(int fd)
{
	struct flock	lock;

	lock.l_type = F_UNLCK;
	lock.l_start = 0;
	lock.l_whence = SEEK_SET;
	lock.l_len = 0;

	fcntl(fd, F_SETLK, &lock);
	close(fd);
}
/*----	TmFunc			----*/
void	TmFunc()
{
}
/*----	GetCookie		----*/
void	GetCookie()
{
	char	*cookie;
	char	*srch;

	if((cookie = getenv("HTTP_COOKIE")) == (char *)0) {
		return;
	}

	/* Cookie tag check	*/
	srch = cookie;
	while(*srch != '\0') {
		if(strncmp(srch, "ChuraCount=", 11) == 0) {
			srch += 11;
			if(strncmp(srch, "SG=", 3) == 0) {
				srch += 3;
				AC = srch;
				GetNextData(srch, ';');
			}
			break;
		} else {
			srch = GetNextData(srch, ';');
			if(*srch == ' ') {
				srch++;
			}
		}
	}
}
/*----	PutCookie		----*/
void	PutCookie(time_t nowtimer)
{
	char *youbi[7] = {	"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
	char *month[12] = {	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	struct tm	*nowtm;

	nowtimer += (60 * 60 * 24 * 100);
	if((nowtm = gmtime(&nowtimer)) != NULL) {
		nowtm->tm_year += 1900;

		printf("Set-Cookie: ChuraCount=SG=%s; expires=%s, %2d-%s-%4d %2d:%2d:%2d GMT\n",
		  AC,
		  youbi[nowtm->tm_wday],
		  nowtm->tm_mday, month[nowtm->tm_mon], nowtm->tm_year,
		  nowtm->tm_hour, nowtm->tm_min, nowtm->tm_sec);
	}
	fflush(stdout);
}
/*----	GetNextData		----*/
char	*GetNextData(char *rdbuf, char cep)
{
	while(*rdbuf != cep && *rdbuf != '\0') {
		rdbuf++;
	}
	if(*rdbuf == cep) {
		*rdbuf = '\0';
		rdbuf++;
	}
	return(rdbuf);
}

